#!/usr/bin/ruby
# encoding: utf-8
#
#Diese Software unterliegt der GPL
#Copyright Karlheinz Klingbeil
#
require 'socket'
require 'optparse'
require 'securerandom'
require 'syslog'

class TcpServer
  def initialize
    log "TCPSERVER 0.1/20.07.2022"
    #Encoding.default_external = Encoding::UTF_8
    #Encoding.default_internal = Encoding::UTF_8
    $stdout = File.new( '/dev/null', 'w' )
    @config = {}
    @config[:host] = "localhost"
    @config[:port] = "9105"
    @config[:dir] = "/tmp"
    @config[:script] = "/data/script/dmc2"
    @config[:print] = false

    OptionParser.new do |opts|
      opts.banner = "Usage: tcpserver [options]"
      opts.on('-p', '--port port', 'port') { |v| @config[:port] = v }
      opts.on('-f', '--host host', 'host') { |v| @config[:host] = v }
      opts.on('-d', '--dir dir', 'directory') { |v| @config[:dir] = v }
      opts.on('-s', '--script name', 'script name') { |v| @config[:script] = v }
      opts.on('-n', '--print', 'print direct') { |v| @config[:print] = v }
    end.parse!

    log "Open Server on #{@config[:host]}:#{@config[:port]}"
    log "Direcory=#{@config[:dir]}"
    log "Script=#{@config[:script]}"
    server = TCPServer.open(@config[:host],@config[:port])    
     loop {                         
      log "start loop" 
       Thread.start(server.accept) do |client|
         log "Accept Connection from #{client.peeraddr}"
         data = client.read()
         @config[:tmpfile] = @config[:tmpfile] = File.join(@config[:dir],SecureRandom.uuid)
         log "Create Temp File :#{@config[:tmpfile]}"
         tmp = File.open(@config[:tmpfile],'w')
         tmp.write(data)
         tmp.close
         if @config[:print]
          `#{@config[:script]} -p -f #{@config[:tmpfile]}`
         else
          `#{@config[:script]} -f #{@config[:tmpfile]}`
        end
       end
     }
     server.close  
   rescue SystemExit, Interrupt
     log "CTRL_C detected"
     server.close
     raise
   end

  def getTimeStamp
    date = Time.new
    return date.strftime('%Y-%m-%d-%H-%M-%S')
  end


  def log (str)
    str = "#{getTimeStamp()} :#{str}\n"
    str = str.sub("%","P")
    puts str
    Syslog.open($0, Syslog::LOG_PID|Syslog::LOG_CONS )
    Syslog.log(Syslog::LOG_PID, str )
    Syslog.close
 end


end

t = TcpServer.new

